package com.jz.dmp.cmdexectool.scheduler.common.task.sync;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.constant.DatasouceTypeConstant;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.entity.DmpSyncingDatasource;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: SyncParameters
 * @Description: sync parameters
 * @Author : Bellamy
 * @Date 2021/2/25
 * @Version 1.0
 */
public class SyncParameters extends AbstractParameters {

    /**
     * shell script
     */
    private String script;

    private String taskAppId;

    /**
     * 数据源相关配置
     */
    private String source;

    /**
     * 目标相关配置
     */
    private String sink;

    /**
     * 环境相关配置
     */
    private String env;

    /**
     * ETL相关配置
     */
    private String transform;

    /**
     * waterdropScript脚本
     */
    private String waterdropScript;

    /**
     * resource list
     */
    private List<ResourceInfo> resourceList;

    public SyncParameters(String script, DmpProjectConfigInfoDto projectConfigInfoDto, DmpSyncingDatasourceDao dmpSyncingDatasourceDao, FreeMarkerConfigurer freeMarkerConfig) {
        source = "";
        env = "";
        sink = "";
        transform = "";

        this.script = script;

        JSONObject scriptObj = JSONObject.parseObject(script);
        Map<String, Object> scriptJson = (Map<String, Object>) scriptObj.get("params");
        Map<String, Object> scripts = (Map<String, Object>) scriptJson.get("scripts");
        Map<String, Object> sourceObj = (Map<String, Object>) scripts.get("reader"); //来源数据
        Map<String, Object> targetObj = (Map<String, Object>) scripts.get("writer"); //目标数据
        Map<String, Object> mappingObj = (Map<String, Object>) scripts.get("mappingRelation"); //字段映射关系
        //evn
        Map<String, String> envModel = new HashMap<String, String>();
        envModel.put("sparkappname", "Waterdrop");
        env = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfig);

        //来源信息
        Integer sourceId = Integer.valueOf((String) sourceObj.get("sourceDbId"));
        DmpSyncingDatasource dmpSyncingDatasource = dmpSyncingDatasourceDao.queryById(sourceId);
        Integer sourceTypeId = dmpSyncingDatasource.getDatasourceType();
        String[] registerTableName = ((String) sourceObj.get("registerTableName")).split(",");

        //目标信息
        Integer targetDbId = Integer.valueOf((String) targetObj.get("targetDbId"));
        DmpSyncingDatasource targetDatasource = dmpSyncingDatasourceDao.queryById(targetDbId);
        Integer targetTypeId = dmpSyncingDatasource.getDatasourceType();

        //Jdbc(MySQL、Oracle、SqlServer、PostgreSQL、Informix、DB2)
        //source
        if (sourceTypeId == DatasouceTypeConstant.MySQL || sourceTypeId == DatasouceTypeConstant.Oracle || sourceTypeId == DatasouceTypeConstant.SqlServer || sourceTypeId == DatasouceTypeConstant.PostgreSQL
                || sourceTypeId == DatasouceTypeConstant.Informix || sourceTypeId == DatasouceTypeConstant.DB2) {
            for (String tableName : registerTableName) {
                Map jdbcModel = new HashMap();
                jdbcModel.put("driver", dmpSyncingDatasource.getDriverClassName());
                jdbcModel.put("url", dmpSyncingDatasource.getJdbcUrl());
                jdbcModel.put("table", tableName);
                jdbcModel.put("result_table_name", tableName);
                jdbcModel.put("user", dmpSyncingDatasource.getUserName());
                jdbcModel.put("password", dmpSyncingDatasource.getPassword());
                //jdbcModel.put("partitionColumn", "");
                //jdbcModel.put("numPartitions", "");
                //jdbcModel.put("lowerBound", dmpSyncingDatasource.getPassword());
                //jdbcModel.put("upperBound", dmpSyncingDatasource.getPassword());
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, jdbcModel, freeMarkerConfig);
            }
        }
        //sink
        if (targetTypeId == DatasouceTypeConstant.MySQL || targetTypeId == DatasouceTypeConstant.Oracle || targetTypeId == DatasouceTypeConstant.SqlServer || targetTypeId == DatasouceTypeConstant.PostgreSQL
                || targetTypeId == DatasouceTypeConstant.Informix || targetTypeId == DatasouceTypeConstant.DB2) {
            Map jdbcSinkModel = new HashMap();
            jdbcSinkModel.put("driver", targetDatasource.getDriverClassName());
            jdbcSinkModel.put("url", targetDatasource.getJdbcUrl());
            jdbcSinkModel.put("save_mode", targetObj.get("targetInsertMergeOverwrite")); //# 存储模式，支持overwrite、append、update、ignore、error
            if (null != targetObj.get("targetInsertMergeOverwrite")) { //当存储模式是 overwrite时，仅清除表中数据
                if ("overwrite".equals(targetObj.get("targetInsertMergeOverwrite"))) {
                    jdbcSinkModel.put("truncate", true);
                } else {
                    jdbcSinkModel.put("truncate", false);
                }
            }
            jdbcSinkModel.put("dbtable", targetObj.get("targetTable")); //目标表
            jdbcSinkModel.put("user", targetDatasource.getUserName());
            jdbcSinkModel.put("password", targetDatasource.getPassword());
            sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_JDBC, jdbcSinkModel, freeMarkerConfig);
        }

        if (sourceTypeId == DatasouceTypeConstant.Elasticsearch) {
            //source
            for (String tableName : registerTableName) {
                List<String> list = new ArrayList<>();
                String hosts = dmpSyncingDatasource.getHost() + ":" + dmpSyncingDatasource.getPort();
                list.add(hosts);
                Map jdbcModel = new HashMap();
                jdbcModel.put("hosts", list);
                jdbcModel.put("result_table_name", tableName);
                //jdbcModel.put("index", "");
                //jdbcModel.put("name_age", "");
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_ELASTICSEARCH, jdbcModel, freeMarkerConfig);
            }
        }
        if (sourceTypeId == DatasouceTypeConstant.Hive) {

        }
        if (sourceTypeId == DatasouceTypeConstant.SFTP) {
            //source
            for (String tableName : registerTableName) {
                Map sftpModel = new HashMap();
                sftpModel.put("host", dmpSyncingDatasource.getHost()); //主机名
                sftpModel.put("user", dmpSyncingDatasource.getUserName()); //用户
                sftpModel.put("password", dmpSyncingDatasource.getPassword());
                sftpModel.put("port", dmpSyncingDatasource.getPort()); //端口
                if (null != sourceObj.get("fileType")) {
                    sftpModel.put("fileType", sourceObj.get("fileType").toString()); //文件类型
                }
                sftpModel.put("delimiter", sourceObj.get("sourceCsvDelimiter")); //分隔符
                sftpModel.put("head", true); //是否加载第一行
                sftpModel.put("multiLine", true); //多行解析
                if (null != sourceObj.get("sourceFtpDir")) {
                    sftpModel.put("path", sourceObj.get("sourceFtpDir").toString()); //文件路径
                }
                sftpModel.put("result_table_name", tableName); //spark生成的临时表名
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_SFTP, sftpModel, freeMarkerConfig);
            }

        }

    }

    @Override
    public boolean checkParameters() {
        return waterdropScript != null && !waterdropScript.isEmpty();
    }

    /**
     * get project resource files list
     *
     * @return resource files list
     */
    @Override
    public List<ResourceInfo> getResourceFilesList() {
        return resourceList;
    }

    public String getScript() {
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    public String getTaskAppId() {
        return taskAppId;
    }

    public void setTaskAppId(String taskAppId) {
        this.taskAppId = taskAppId;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getSink() {
        return sink;
    }

    public void setSink(String sink) {
        this.sink = sink;
    }

    public String getEnv() {
        return env;
    }

    public void setEnv(String env) {
        this.env = env;
    }

    public String getTransform() {
        return transform;
    }

    public void setTransform(String transform) {
        this.transform = transform;
    }

    public String getWaterdropScript() {
        return waterdropScript;
    }

    public void setWaterdropScript(String waterdropScript) {
        this.waterdropScript = waterdropScript;
    }

    public List<ResourceInfo> getResourceList() {
        return resourceList;
    }

    public void setResourceList(List<ResourceInfo> resourceList) {
        this.resourceList = resourceList;
    }
}
